-- by mor2000

-- push object

PushObjectListener = {}

PushObjectListener.RAYCASTDISTANCE = 2
PushObjectListener.IMPULSEPERTON = 2.5

function PushObjectListener:update(dt)
	-- gamepad input
	if getGamepadEnabled() then
		if self:isPlayerActive() then
			local gamepadButtonPressed = false
			for gamepadDevice=1,getNumOfGamepads() do
				local gamepadButtons = InputBinding.actions[InputBinding.PUSHOBJECT_PUSH].gamepadButtons
				if InputBinding.areGamepadButtonsPressed(gamepadDevice, gamepadButtons) then
					if not self.doRaycastGamepad then
						self.doRaycastGamepad = true
						gamepadButtonPressed = true
						self:doRaycastForObject(self:getCameraPositionAndRotation())
						break
					end
				end
			end
			if not gamepadButtonPressed then
				self.doRaycastGamepad = false
			end
		end
	end
end

function PushObjectListener:mouseEvent(posX, posY, isDown, isPressed, button)
	-- check if mouse button is pressed
	if self:isPlayerActive() then
		if isPressed then
			if not self.doRaycastMouse then
				self.doRaycastMouse = true
				self:doRaycastForObject(self:getCameraPositionAndRotation())
			end
		else
			if self.doRaycastMouse then
				self.doRaycastMouse = false
			end
		end
	end
end

function PushObjectListener:keyEvent(unicode,sym,modifier,isDown)
	-- key pressed
	if self:isPlayerActive() then
		if InputBinding.isPressed(InputBinding.PUSHOBJECT_PUSH) then
			if not self.doRaycastKey then
				self.doRaycastKey = true
				self:doRaycastForObject(self:getCameraPositionAndRotation())
			end
		else
			if self.doRaycastKey then
				self.doRaycastKey = false
			end
		end
	end
end

function PushObjectListener:isPlayerActive()
	return g_gui.currentGui==nil and
			g_currentMission.controlPlayer and
			g_currentMission.player~=nil and
			g_currentMission.player.currentToolId==0
end

function PushObjectListener:getCameraPositionAndRotation()
	if self:isPlayerActive() then
		local camx, camy, camz = getWorldTranslation(g_currentMission.player.cameraId)
		local dirx, diry, dirz = localDirectionToWorld(g_currentMission.player.cameraId, 0, 0, -1)
		local rotY = (g_currentMission.player.rotY+math.pi) % (2*math.pi) - math.pi
		return camx, camy, camz, dirx, diry, dirz, rotY
	end
end

function PushObjectListener:doRaycastForObject(camx, camy, camz, dirx, diry, dirz, rotY)
	if camx==nil or camy==bil or camz==nil or dirx==nil or diry==nil or dirz==nil or rotY==nil then
		return false
	end
	
	if g_server~=nil then
		self.objectToPush = {}
		raycastAll(camx, camy, camz, dirx, diry, dirz, "raycastForObjectCallback", PushObjectListener.RAYCASTDISTANCE, self)
		if self.objectToPush.shapeId~=nil then
			local objectMass = getMass(self.objectToPush.shapeId)
			local impulse = math.sqrt(PushObjectListener.IMPULSEPERTON * objectMass)
			local partX, partZ = -math.sin(rotY), -math.cos(rotY)
			local partSum = math.abs(partX) + math.abs(partZ)
			local impulseX = impulse * partX/partSum
			local impulseY = impulse * 0.1
			local impulseZ = impulse * partZ/partSum
			addImpulse(self.objectToPush.shapeId,impulseX,impulseY,impulseZ,self.objectToPush.x,self.objectToPush.y,self.objectToPush.z,false)
		end
	else
		-- do raycast on server
		PushObjectRaycastEvent.sendEvent(camx, camy, camz, dirx, diry, dirz, rotY)
	end
end

function PushObjectListener:raycastForObjectCallback(shapeId, x, y, z, distance)
	if self:getIsObject(shapeId) or self:getIsLog(shapeId) then
		self.objectToPush.shapeId = shapeId
		self.objectToPush.x = x
		self.objectToPush.y = y
		self.objectToPush.z = z
		return false
	end
	return true
end

function PushObjectListener:getIsObject(shapeId)
	local object = g_currentMission.nodeObjects[shapeId] or
					g_currentMission.objectToTrailer[shapeId] or
					g_currentMission.nodeToVehicle[shapeId]
	if object~=nil and object.isa~=nil and object:isa(Object) then
		return true
	end
	return false
end

function PushObjectListener:getIsLog(shapeId)
	local splitType = SplitUtil.splitTypes[getSplitType(shapeId)]
	if splitType~=nil then
		return true
	end
	return false
end

local function emptyFunc() end
PushObjectListener.loadMap = emptyFunc
PushObjectListener.deleteMap = emptyFunc
PushObjectListener.draw = emptyFunc

addModEventListener(PushObjectListener)
